/*
    Copyright (c) 2010, 2011 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of dS Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.
*/

#ifndef __UTIL_H__
#define __UTIL_H__

#include <string>
#include <list>
#include <map>
#include <time.h>
#include <sys/time.h>

#include "hwdesc.h"

namespace dSA
{

// http://stackoverflow.com/questions/4108313/how-do-i-find-the-length-of-an-array
template<class T, size_t N>
size_t array_size(T (&)[N]) { return N; }

struct util
{
    static std::string str(int i);
    static std::string str(uint32_t u);
    static std::string str(unsigned long l);
    static std::string str(long long int i);
    static std::string str(dsid_t dsid);
    static std::string str(dsuid_t dsuid);
    static std::string str(time_t t);
    static std::string hex(int i, int width = 8);
    static std::string hex(uint32_t l, int width = 8);
    static std::string hex(uint64_t l, int width = 16);
    static std::string hex(uint8_t *data, int length);
    static int str2int(std::string str);
    static unsigned long str2ulong(std::string str);
    static uint64_t str2uint64_t(std::string str);
    static uint32_t str2uint32_t(std::string str);
    static uint16_t str2uint16_t(std::string str);
    static long long int str2llint(std::string str);
    static double str2double(std::string str);
    static dsid_t str2dsid(std::string dsid_str);
    static dsuid_t str2dsuid(std::string dsuid_str);
    static uint32_t dsuid2lower(const dsuid_t& dsuid);
    static bool dsuid2dsid(const dsuid_t dsuid, dsid_t *dsid);
    static time_t str2time_t(std::string str);
    static std::string epoch2local(time_t t);
    static std::string mac2str();
    static std::string binaryData2str(uint16_t data);
    static void strsplit(const std::string& str, char d, std::list<std::string>& list);
    static std::string autosprintf(const char* format, ...);
    static std::string render_display_id(dsuid_t dsuid,
                                         std::string hw_guid = "");
};

struct webutil
{
    typedef std::map<const std::string, std::string> query;

    static std::string url_unescape(const std::string& url);
    static std::string url_escape(const std::string& url);
    static std::string command_from_uri(const std::string& uri);
    static void parse_query(const std::string& query_string, query &parameters);
    static std::string construct_query(const query& q);
    static std::string http_code_msg(const int code);
    static std::string render_http_header(const int code,
                                          const std::string& content_type,
                                          const std::string& attachment = "");
};

struct fileutil
{
    static bool exists(std::string file, bool need_dir = false);
    static size_t fsize(std::string file);
    static std::vector<std::string> glob(std::string const &filename, int flags);
};

struct netutil
{
    static std::string if_to_ip(std::string interface);
};

struct timeutil
{
    static std::string epoch2local(time_t t);
    static std::string localizedEpoch2local(time_t t);
    static int compareTimespecs(struct timespec *a,  struct timespec *b);
    static void getTimespecNow(struct timespec *ts);
    static void getTimespecAfterMillis(uint64_t delta, struct timespec *ret,
                                       struct timespec *start = NULL);
};

struct ds
{
    static bool is_standard_group(int group);
};

}; // namespace

#endif//__UTIL_H__
